% This code shows a demo of the linear separability of signal classes
% in the CDT space. 
% Written by Soheil kolouri, August 2014

clear
clc
addpath('Codes')
load('Data/TextureData.mat')
eps=1e-5;
%Generate 1D signals to be the histograms of a texture dataset with two
%classes.
for k=1:size(I,3)
    J(:,k)=imhist(I(:,:,k));
    J(:,k)=J(:,k)/sum(J(:,k))+eps;        
end
%Calculate the template signal to be the average
J0=mean(J')';
for k=1:size(I,3)
    JCDT(:,k)=CDT(J0,J(:,k));   
end

%Calculate the discriminant subspaces in the signal space and in the image 
%space.
[W] = PLDA(J,label,1e-2,2);
[WCDT] = PLDA(JCDT,label,1e-2,2);

Wproj=W'*J;
WCDTproj=WCDT'*JCDT;
subplot(1,2,1)
plot(Wproj(1,label==0),Wproj(2,label==0),'xr','markersize',20);
hold on
plot(Wproj(1,label==1),Wproj(2,label==1),'ok','markersize',20);
legend('Class 1','Class 2')
title('Discriminant subspace in the signal space','fontsize',24)
set(gca,'fontsize',24)
subplot(1,2,2)
plot(WCDTproj(1,label==0),WCDTproj(2,label==0),'xr','linewidth',2,'markersize',20);
hold on
plot(WCDTproj(1,label==1),WCDTproj(2,label==1),'ok','linewidth',2,'markersize',20);
legend('Class 1','Class 2')
title('Discriminant subspace in the CDT space','fontsize',24)
set(gca,'fontsize',24)



